@@ -317,6 +317,8 @@ load_meta_themes (GtkTreeView *tree_view,
 
   current_gtk_theme = gconf_client_get_string (client, GTK_THEME_KEY, NULL);
   current_icon_theme = gconf_client_get_string (client, ICON_THEME_KEY, NULL);
+  g_object_unref (client);
+
   window_manager = gnome_wm_manager_get_current (gdk_display_get_default_screen (gdk_display_get_default ()));
   wm_settings.flags = GNOME_WM_SETTING_THEME;
 
@@ -577,6 +579,8 @@ meta_theme_selection_changed (GtkTreeSelection *selection,
     /* Get the settings */
     current_gtk_theme = gconf_client_get_string (client, GTK_THEME_KEY, NULL);
     current_icon_theme = gconf_client_get_string (client, ICON_THEME_KEY, NULL);
+    g_object_unref (client);
+
     window_manager = gnome_wm_manager_get_current (gdk_display_get_default_screen (gdk_display_get_default ()));
     wm_settings.flags = GNOME_WM_SETTING_THEME;
     if (window_manager) {
@@ -741,7 +745,7 @@ remove_custom_row_from_meta_theme (void)
   custom_meta_theme_info.name = NULL;
 }
 
-gboolean
+static gboolean
 themes_equal (GnomeThemeMetaInfo *a, GnomeThemeMetaInfo *b)
 {
   if (!a->gtk_theme_name ||
@@ -785,6 +789,8 @@ update_settings_from_gconf_idle (gpointer data)
   /* Get the settings */
   current_gtk_theme = gconf_client_get_string (client, GTK_THEME_KEY, NULL);
   current_icon_theme = gconf_client_get_string (client, ICON_THEME_KEY, NULL);
+  g_object_unref (client);
+
   window_manager = gnome_wm_manager_get_current (gdk_display_get_default_screen (gdk_display_get_default ()));
   wm_settings.flags = GNOME_WM_SETTING_THEME;
   if (window_manager) {
@@ -919,7 +925,19 @@ static void
 window_settings_changed (GnomeWindowManager *window_manager,
 			 GladeXML           *dialog)
 {
-  /* There is no way to know what changed, so we just check it all */
+  static gchar *window_theme = NULL;
+  GnomeWMSettings wm_settings;
+
+  wm_settings.flags = GNOME_WM_SETTING_THEME;
+  gnome_window_manager_get_settings (window_manager, &wm_settings);
+  if (window_theme == NULL || strcmp (wm_settings.theme, window_theme) != 0)
+    {
+      g_free (window_theme);
+      window_theme = g_strdup (wm_settings.theme);
+    }
+  else
+    return;
+
   update_settings_from_gconf ();
   gnome_theme_details_update_from_gconf ();
 }
@@ -966,7 +984,7 @@ update_font_button_state (GladeXML *dialog)
 
       g_free (str);
     }  
-  
+  g_object_unref (client);  
 }
 
 static void
@@ -1070,7 +1088,7 @@ static void
 gnome_meta_theme_installer_run_cb (GtkWidget *button,
 				   GtkWidget *parent_window)
 {
-  gnome_theme_installer_run (parent_window, NULL, FALSE);
+  gnome_theme_installer_run (parent_window, NULL);
 }
 
 
@@ -1113,6 +1131,7 @@ apply_font_clicked (GtkWidget *button,
 
       gconf_client_set_string (client, FONT_KEY, meta_theme_info->application_font, NULL);
     }  
+  g_object_unref (client);
 }
 
 static void
@@ -1185,6 +1204,8 @@ setup_dialog (GladeXML *dialog)
 			   FONT_KEY,
 			   (GConfClientNotifyFunc) &font_key_changed,
 			   dialog, NULL, NULL);
+  g_object_unref (client);
+
   if (window_manager)
     g_signal_connect (G_OBJECT (window_manager),
 		      "settings_changed",
@@ -1388,7 +1409,7 @@ gnome_theme_manager_drag_data_received_cb (GtkWidget *widget, GdkDragContext *co
 		gnome_vfs_uri_list_unref (uris);
 	}
 
-	gnome_theme_installer_run (widget, filename, FALSE);
+	gnome_theme_installer_run (widget, filename);
 	g_free (filename);
 }
 
@@ -1403,6 +1424,7 @@ get_default_string_from_key (const char *key)
 
   client = gconf_client_get_default ();
   value = gconf_client_get_default_from_schema (client, key, &error);
+  g_object_unref (client);
 
   if (error)
     {
